#! /usr/bin/perl -W

# Auteur : B. Marchal (Benoit.Marchal@univ-lorraine.fr)
# Date   : 30 janvier 2015

# les chaînes entre < xxx > sont à remplacer par les bonnes valeurs.

=encoding utf-8

=head1 Name

rappelCle - rappel des demandes non répondues pour les clefs d'authentification et de modération

=head1 Synopsys

rappelCle [--notest] [--polling <periode> ]

=head1 Description

On va parcourir le répertoire de spool de demande d'authentification (resp de modération) pour rappeler à l'émetteur de valider son message (resp la modération)

Si un message est en attente d'authentification, on rappelle l'émetteur au bout du temps de polling et 24 heures après

Si un message est en attente de modération, on ne prévient que 24 heures après.

=over 8

=item B<--notest>

On met en production

=item B<--polling periode>

Définition du temps de polling : 1h = une heure, 1j = un jour, ...

=back

=cut 

use strict;
use English;
use Getopt::Long;
use MIME::Lite;
use MIME::Base64;

##### Debut des paramètres #####

my $DEFAULTPOLLING = '1h';	# définit à la fois le début du polling  et sa durée (si on met 1h, on prend les messages entre -1h et -2h, ainsi que les messages de la veille entre j-1-1h et j-1-2h)
my $DEFAULTUNIT    = 'h';	# unite de temps

my $addrSympa	= 'sympa@<mon domaine>';		# Adresse de l'application (champ FROM des messages aux utilisateurs
my $addrSympaSimple = 'sympa';				# Adresse de l'application (sans le domaine)
my $addrAdmin	= '<adresse Administrateur>@<mon domaine>';	# Adresse de l'administrateur de l'application

my @adressesIgnorees	= ('MAILER-DAEMON@<mon domaine>' );	# liste des Adresses à ignorer
my @domainesAutorises	= ('<mon robot1>', '<mon robot2>', '<mon robot3>');	 # Listes des domaines pris en compte (il peut y en avoir autant que l'on veut, on les sépare par des ','

my $formatCommandeSuppression = "/bin/rm %s";	# Commande pour supprimer un fichier

my $dirLog = '/var/log';			# Répertoire des logs
my $ficLog = "$dirLog/sympa.log";		# nom du fichier de log

my $sympa_conf="/etc/sympa/sympa.conf";		# fichier de coniguration de SYMPA

##### Fin des paramètres #####

my $ficLogAnc = rechercheFichierLogPrecedent (); # Recherche du fichier de log de la veille

my ($polling, $debug, $notest);
GetOptions(
    'polling:s'	=> \$polling,
    'debug!'	=> \$debug,
    'notest!'   => \$notest, 
);

# Récupération de la configuration de SYMPA

my $queueauth = `grep "^queueauth" $sympa_conf`; $queueauth =~ s/queueauth\s+(.*)\s*$/$1/;
my $queuemod  = `grep "^queuemod" $sympa_conf`;  $queuemod  =~ s/queuemod\s+(.*)\s*$/$1/;

# On prend des valeurs par défaut
$queueauth = "/var/spool/sympa/auth"       if ( !defined($queueauth) or $queueauth eq "" ) ;
$queuemod  = "/var/spool/sympa/moderation" if ( !defined($queuemod)  or $queuemod  eq "" ) ;


my $interval = conversionTemps ($polling || $DEFAULTPOLLING);

my @rapport = ();
analyseSpool($queueauth) ;
analyseSpool($queuemod) ;

if (scalar(@rapport) ) {
    my $texte = join "<br>", @rapport;
    envoiMessage($addrSympa,$addrAdmin,$addrSympa, 'Rappel Sympa', $texte,"");
}

printf STDERR "Interval = %f\n", $interval if ($debug);
printf STDERR "Log courant   = %s\n", $ficLog if ($debug);
printf STDERR "Log précédent = %s\n", $ficLogAnc if ($debug);

sub analyseSpool {
    my $queue = shift;
    my @fichiers;
    # Analyse du spool d'authentification

    printf STDERR " Analyse de la file %s\n", $queue if ($debug);

    my $dirOrigine = chdir $queue or die "Impossible de se positionner dans le répertoire $queue\n";
    opendir SPOOL, '.' or die "Impossible d'ouvrir le répertoire $queue\n";
    @fichiers = grep !/^\./, readdir SPOOL;
    closedir SPOOL;

    for my $f (@fichiers ) {
	####printf STDERR "fichier : %s -> ", $f if ($debug);
	# Recupération des caractéristiques du fichiers
	next if ( ! -f $f ); 			# On passe au fichier suivant si celui-ci a été supprimé le temps du traitement des fichiers précédents
	my ($liste,$cle) = split /_/, $f;
	my @stat = stat $f;
	my $age = -M $f;

	my $recherche = "";
	my ($premierTerme,$domaine) = split /\@/, $liste;
	$recherche = 'info Commands::confirm.*CONFIRM '.$cle.' from\s\S*\sfor list '.$premierTerme.' accepted' if ( $queue eq $queueauth );
	$recherche = 'notice Commands::parse.*CONFIRM '.$cle.'$' if ( $queue eq $queueauth );
	$recherche = 'info Commands::distribute.*DISTRIBUTE '.$premierTerme.' '.$cle.' from .*\@.* accepted' if ( $queue eq $queuemod );

	if ( $recherche ne "" ) {
	    # On commence par regarder dans les journaux si la personne n'aurait pas validé.
	    printf STDERR "Recherche demandée : %s\n", $recherche if ($debug);
	    my $retour = `grep "$recherche" $ficLog $ficLogAnc 2>/dev/null | head -1`; chomp ($retour);
	    printf STDERR "Retour : %s\n", $retour if ($debug);
	    if ( $retour =~ /Commands/ ) {
		# On détruit le fichier
		if ( $notest ) {
		    my $command = sprintf $formatCommandeSuppression, $f;
		    system("$command");
		} else {
		    push @rapport,  "SIMUL : ";
		}
		push @rapport,  "	Supression du fichier $f\n";
		next; # On passe au fichier suivant
	    }

	    my $rappel = 0;

	    $rappel = 1    if ( ($age > $interval)       &&  ($age <= 2 * $interval) && $queue eq $queueauth ); # On ne positionne pas de rappel n°1 pour les messages de modération
	    $rappel = 2    if ( ($age > ($interval + 1)) &&  ($age <= (2 * $interval) + 1)); # On est 24h plus tard

	    if ( $rappel ) {
	    	printf STDERR "Fichier : %s\n", $f if ($debug);
		# On lance un rappel, on analyse le message pour connaître à qui envoyer le message
 
		printf STDERR "	Rappel : %s\n", $rappel if ($debug);

		# On analyse le fichier pour chercher l'origine du message. On prioritise par le return-path, sinon le from
		my $origine = `grep '^Return-Path:' $f | cut -f2 -d ' ' | head -1 `; chomp $origine; 
		if ( ! $origine  ) {
		    $origine = `grep '^From:' $f | cut -f2 -d ' ' | head -1`; chomp $origine; 
		}
		$origine =~ s/.*<(.*)>.*/$1/;

		# Recherche de l'adresse (fonction du type de message)
		my ($emetteur, $adresse, $adresseDeRetour, $addrSympaRetour, $complementSujet, $texte, $fichierJoint);
		# On recherche les paramètres pour les messages d'authentification
		if ( $queue eq $queueauth ) {
		    $adresseDeRetour = sprintf "%s-owner@%s",   $premierTerme, $domaine;
		    $adresse = $origine;
		    printf STDERR "	adresse déterminée : %s\n", $adresse if ($debug);
		    $emetteur = $addrSympa;
		    $complementSujet = "Confirmation d'un message";
		    $addrSympaRetour = sprintf "%s@%s", $addrSympaSimple, $domaine;
		    $texte = "
Bonjour,
<p>
Vous avez envoy&eacute; un message &agrave; la liste $liste, liste qui demande une authentification. <br>
En retour, le gestionnaire de listes vous a adress&eacute; un message demandant la confirmation de cet envoi.
<p>
Sauf erreur, celui-ci n'a rien re&ccedil;u de votre part.
<p>
Pour confirmer votre envoi, vous pouvez cliquer sur le lien suivant mailto:$addrSympaRetour?subject=CONFIRM%20$cle
<p>
ou envoyer un message &agrave; $addrSympaRetour avec comme objet :
<p>
CONFIRM $cle
<p>
Cordialement
<p>
PS : Ceci est un message automatique, ne pas y r&eacute;pondre";
		    $fichierJoint="";
		}

		# On recherche les paramètres pour les messages de modération
		if ( $queue eq $queuemod ) {
		    $emetteur 	 = sprintf "%s-request@%s", $premierTerme, $domaine;
		    $adresseDeRetour = sprintf "%s-owner@%s",   $premierTerme, $domaine;
		    $adresse 	 = sprintf "%s-editor@%s",  $premierTerme, $domaine;
###		    $adresse 	 = sprintf "%s-editor@%s",  "testbenoit", "univ-lorraine.fr";
		    printf STDERR "	adresse déterminée : %s\n", $adresse if ($debug);
		    $complementSujet = sprintf "Message à modérer de %s pour la liste %s",$origine,$premierTerme;
		    $addrSympaRetour = sprintf "%s@%s", $addrSympaSimple, $domaine;
		    $texte = "
Bonjour,
<p>
Un message &agrave; mod&eacute;rer vous a &eacute;t&eacute envoy&eacute; pour la liste $premierTerme.
<p>
Sauf erreur, vous n'avez ni valid&eacute;, ni invalid&eacute; celui-ci.
<p>&nbsp;<p>
Pour diffuser le message ci-joint dans la liste $liste, vous pouvez cliquer sur le lien suivant : mailto:$addrSympaRetour?subject=DISTRIBUTE%20$premierTerme%20$cle
<br>
ou envoyez un message &agrave; $addrSympaRetour avec comme objet : 
<br>
DISTRIBUTE $premierTerme $cle
<p>


Pour refuser sa diffusion (il sera effac&eacute;), cliquez ici : mailto:$addrSympaRetour?subject=REJECT%20$premierTerme%20$cle
<br>
ou envoyez un message &agrave; $addrSympaRetour avec comme objet :
<br>
REJECT $premierTerme $cle

<p>
Cordialement
<p>
PS : Ceci est un message automatique, ne pas y r&eacute;pondre";
		    $fichierJoint = $f;
		}	
		next if ( ! $adresse );	# on n'a pas réussi à déterminer une adresse on passe au fichier suivant


		my $sujet = (($rappel == 1)?"Premier" : "Dernier")." rappel : $complementSujet";

		envoiMessage($emetteur,$adresse,$adresseDeRetour, $sujet, $texte,$fichierJoint);
	    }
	}
	chdir $dirOrigine;
    }
}

sub envoiMessage {
    my $emetteur 	= shift;
    my $adresse 	= shift;
    my $adresseDeRetour = shift;
    my $sujet   	= shift;
    my $texte   	= shift;
    my $message		= shift;

    printf STDERR "Adresse recue = %s \n", $adresse if ($debug);
    $adresseDeRetour = $adresse 	 = $addrAdmin if ($debug);

    return if ( grep /^$adresse$/, @adressesIgnorees);	# On passe l'émission du message si on doit ignorer cette adresse

    my ($nom,$domaine) = split /@/, $adresse;
    printf STDERR "Adresse = %s => (%s,%s)\n", $adresse, $nom, $domaine if ($debug);
    return if ( $domaine and ! grep /$domaine/, @domainesAutorises ); # On ne renvoit une demande que sur les domaines autorisés
    printf STDERR "  => on envoie\n"  if ($debug);

    if ( $notest ) {
	# Envoi d'un message
	my ($msg) = new MIME::Lite (
	    'From'		=> $emetteur,
	    'To'		=> $adresse,
	    'Return-Path'	=> $adresseDeRetour,
	    'Subject'		=> $sujet,
	    'Type'		=> 'multipart/related',
	    );
	$msg->attr("content-type.charset" => "utf8");
	$msg->attach(
	    'Type'	=> 'TEXT/html',
	    'Encoding'	=> 'quoted-printable',
	    'Data'	=> $texte);

	if ( $message ) {
	    $msg->attach(
		'Type'		=> 'message/rfc822',
		'Encoding'	=> '8bit',
		'Id'		=> "Message original",
		'Path'		=> $message
		);
	}
	$msg->send ;
    } else {
	printf STDERR "Sujet : %s\n", $sujet if ($debug);
	printf STDERR "Texte : %s\n", $texte if ($debug);
	push @rapport,  "SIMUL : " 
    }
    push @rapport,  "	Envoi un message &agrave; l'adresse : $adresse";
}



sub conversionTemps {
    my $int = shift;

    my $unit = $DEFAULTUNIT;
    my $duree;

    printf STDERR "Interval demandé : %s\n", $int if ($debug);
    # recupération de l'unité de temps (dernier caractère du paramètre si texte)
    if ( $int =~ /^(\d+)([hm])$/ ) {
	$unit = $2 ;
    } else {
	die "l'unité de temps de polling n'est pas précisée ou bien n'est pas valide : [h]";
    }

    $duree = $1 / 24 		if ($unit eq 'h'); # on est en heure
    $duree = $1 / (24 * 60) 	if ($unit eq 'm'); # on est en minute

    return $duree;
}
    
sub rechercheFichierLogPrecedent {
    my $dateAnt= `date +"%Y%m%d"`; chomp $dateAnt;	# Date de la veille
    if ( -f "$ficLog-$dateAnt" )  {
	return "$ficLog-$dateAnt";
    } else {
	$dateAnt= `date --date ' 1 days ago' +"%Y%m%d"`; chomp $dateAnt;
	if ( -f "$ficLog-$dateAnt" )  {
	    return "$ficLog-$dateAnt";
	}
    }
    # On n'a rien trouvé
    return "";
}

